#! /usr/bin/env python3
# -*- coding: utf-8 -*-

from ikabot.helpers.process import run
from ikabot.helpers.gui import *

def update(s):
	out = run('python3 -m pip install --upgrade ikabot').read().decode("utf-8") 
	if 'up-to-date' in out:
		print('\nEstá actualizado')
	else:
		clear()
		print('Actualizando...\n')
		print(out)
		print('Listo.')
		print('Reinicie ikabot para que los cambios surjan efecto.')
	enter()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import os
from ikabot.config import *
from ikabot.web.getSesion import getSesion
from ikabot.helpers.gui import *
from ikabot.helpers.pedirInfo import read
from ikabot.helpers.process import run
from ikabot.helpers.signals import setSignalsHandlers
from ikabot.funcion.subirEdificio import subirEdificios
from ikabot.funcion.menuRutaComercial import menuRutaComercial
from ikabot.funcion.enviarVino import enviarVino
from ikabot.funcion.getStatus import getStatus
from ikabot.funcion.donar import donar
from ikabot.funcion.buscarEspacios import buscarEspacios
from ikabot.funcion.entrarDiariamente import entrarDiariamente
from ikabot.funcion.alertarAtaques import alertarAtaques
from ikabot.funcion.botDonador import botDonador
from ikabot.funcion.update import update
from ikabot.funcion.alertarPocoVino import alertarPocoVino
from ikabot.funcion.comprarRecursos import comprarRecursos

def menu(s):
	banner()
	menu_actions = [
					subirEdificios,
					menuRutaComercial,
					enviarVino,
					getStatus,
					donar,
					buscarEspacios,
					entrarDiariamente,
					alertarAtaques,
					botDonador,
					alertarPocoVino,
					comprarRecursos,
					update
					]
	mnu="""
(0)  Salir
(1)  Lista de construcción
(2)  Enviar recursos
(3)  Enviar vino
(4)  Estado de la cuenta
(5)  Donar
(6)  Buscar espacios nuevos
(7)  Entrar diariamente
(8)  Alertar ataques
(9)  Bot donador
(10) Alertar poco vino
(11) Comprar recursos
(12) Actualizar Ikabot"""
	print(mnu)
	entradas = len(menu_actions)
	eleccion = read(min=0, max=entradas)
	if eleccion != 0:
		try:
			menu_actions[eleccion - 1](s)
		except KeyboardInterrupt:
			pass
		menu(s)
	else:
		clear()

def inicializar():
	os.chdir(os.getenv("HOME"))
	if not os.path.isfile(cookieFile):
		open(cookieFile, 'w')
		os.chmod(cookieFile, 0o600)
	if not os.path.isfile(telegramFile):
		open(telegramFile, 'w')
		os.chmod(telegramFile, 0o600)

def start():
	inicializar()
	s = getSesion()
	setSignalsHandlers(s)
	try:
		menu(s)
	finally:
		if os.fork() == 0:
			s.logout()

def main():
	try:
		start()
	except KeyboardInterrupt:
		clear()

if __name__ == '__main__':
	main()



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import traceback
import json
import math
import re
from decimal import *
from ikabot.helpers.process import forkear
from ikabot.helpers.varios import addPuntos
from ikabot.helpers.gui import enter, banner
from ikabot.helpers.getJson import getCiudad
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.planearViajes import esperarLlegada
from ikabot.helpers.pedirInfo import getIdsDeCiudades, read
from ikabot.config import *
from ikabot.helpers.botComm import *

def asignarRecursoBuscado(s, ciudad, recurso=None):
	if recurso is None:
		print('¿Qué tipo de recurso quiere comprar?')
		for indice, bien in enumerate(tipoDeBien):
			print('({:d}) {}'.format(indice+1, bien))
		recurso = read(min=1, max=5) - 1
		if recurso == 0:
			recurso = 'resource'
	data = {
	'cityId': ciudad['id'],
	'position': ciudad['pos'],
	'view': 'branchOffice',
	'activeTab': 'bargain',
	'type': 444,
	'searchResource': recurso,
	'range': ciudad['rango'],
	'backgroundView' : 'city',
	'currentCityId': ciudad['id'],
	'templateView': 'branchOffice',
	'currentTab': 'bargain',
	'actionRequest': s.token(),
	'ajax': 1
	}
	rta = s.post(payloadPost=data)
	return recurso

def getStoreHtml(s, ciudad):
	url = 'view=branchOffice&cityId={}&position={:d}&currentCityId={}&backgroundView=city&actionRequest={}&ajax=1'.format(ciudad['id'], ciudad['pos'], ciudad['id'], s.token())
	data = s.post(url)
	json_data = json.loads(data, strict=False)
	return json_data[1][1][1]

def obtenerOfertas(s, ciudad):
	html = getStoreHtml(s, ciudad)

	hits = re.findall(r'short_text80">(.*?) *<br/>\((.*?)\)\s *</td>\s *<td>(\d+)</td>\s *<td>(.*?)/td>\s *<td><img src="skin/resources/icon_(\w+)\.png[\s\S]*?white-space:nowrap;">(\d+)\s[\s\S]*?href="\?view=takeOffer&destinationCityId=(\d+)&oldView=branchOffice&activeTab=bargain&cityId=(\d+)&position=(\d+)&type=(\d+)&resource=(\w+)"', html)
	ofertas = []
	for hit in hits:
		oferta = {
		'ciudadDestino': hit[0],
		'jugadorAComprar' : hit[1],
		'bienesXminuto': int(hit[2]),
		'cantidadDisponible': int(hit[3].replace(',', '').replace('<', '')),
		'tipo': hit[4],
		'precio': int(hit[5]),
		'destinationCityId': hit[6],
		'cityId': hit[7],
		'position': hit[8],
		'type': hit[9],
		'resource': hit[10]
		}
		ofertas.append(oferta)
	return ofertas

def calcularCosto(ofertas, cantidadAComprar):
	costoTotal = 0
	for oferta in ofertas:
		if cantidadAComprar == 0:
			break
		comprar = oferta['cantidadDisponible'] if oferta['cantidadDisponible'] < cantidadAComprar else cantidadAComprar
		cantidadAComprar -= comprar
		costoTotal += comprar * oferta['precio']
	return costoTotal

def getOro(s, ciudad):
	url = 'view=finances&backgroundView=city&currentCityId={}&templateView=finances&actionRequest={}&ajax=1'.format(ciudad['id'], s.token())
	data = s.post(url)
	json_data = json.loads(data, strict=False)
	oro = json_data[0][1]['headerData']['gold']
	return int(oro.split('.')[0])


def comprarRecursos(s):
	banner()

	ids = getIdsDeCiudades(s)[0]
	ciudades_comerciales = []
	for idCiudad in ids:
		html = s.get(urlCiudad + idCiudad)
		ciudad = getCiudad(html)
		esComercial = False
		for pos, edificio in enumerate(ciudad['position']):
			if edificio['building'] == 'branchOffice':
				ciudad['pos'] = pos
				html = getStoreHtml(s, ciudad)
				rangos = re.findall(r'<option.*?>(\d+)</option>', html)
				ciudad['rango'] = max(rangos)

				ciudades_comerciales.append(ciudad)
				break

	if len(ciudades_comerciales) == 0:
		print('No hay una Tienda contruida')
		enter()
		return

	ciudad = ciudades_comerciales[0] # por ahora solo uso la primera ciudad

	recurso = asignarRecursoBuscado(s, ciudad)
	banner()

	ofertas = obtenerOfertas(s, ciudad)

	if len(ofertas) == 0:
		print('No se encontraron ofertas.')
		return

	precio_total   = 0
	cantidad_total = 0
	for oferta in ofertas:
		cantidad = oferta['cantidadDisponible']
		unidad   = oferta['precio']
		costo    = cantidad * unidad
		print('cantidad :{}'.format(addPuntos(cantidad)))
		print('precio   :{:d}'.format(unidad))
		print('costo    :{}'.format(addPuntos(costo)))
		print('')
		precio_total += costo
		cantidad_total += cantidad
	print('Total disponible para comprar: {}, por {}\n'.format(addPuntos(cantidad_total), addPuntos(precio_total)))
	cantidadAComprar = read(msg='¿Cuánta cantidad comprar? ', min=0, max=cantidad_total)
	if cantidadAComprar == 0:
		return

	oro = getOro(s, ciudad)
	costoTotal = calcularCosto(ofertas, cantidadAComprar)

	print('\nOro actual : {}.\nCosto total: {}.\nOro final  : {}.'. format(addPuntos(oro), addPuntos(costoTotal), addPuntos(oro - costoTotal)))
	print('¿Proceder? [Y/n]')
	rta = read(values=['y', 'Y', 'n', 'N', ''])
	if rta.lower() == 'n':
		return

	print('Se comprará {}'.format(addPuntos(cantidadAComprar)))
	enter()

	forkear(s)
	if s.padre is True:
		return

	info = '\ninfo sobre el proceso\n'
	setInfoSignal(s, info)
	try:
		do_it(s, ciudad, ofertas, cantidadAComprar, recurso)
	except:
		msg = 'Error en:\n{}\nCausa:\n{}'.format(info, traceback.format_exc())
		sendToBot(msg)
	finally:
		s.logout()

def buy(s, ciudad, oferta, cantidad):
	barcos = int(math.ceil((Decimal(cantidad) / Decimal(500))))
	data = {
	'action': 'transportOperations',
	'function': 'buyGoodsAtAnotherBranchOffice',
	'cityId': oferta['cityId'],
	'destinationCityId': oferta['destinationCityId'],
	'oldView': 'branchOffice',
	'position': ciudad['pos'],
	'avatar2Name': oferta['jugadorAComprar'],
	'city2Name': oferta['ciudadDestino'],
	'type': 444,
	'activeTab': 'bargain',
	'transportDisplayPrice': 0,
	'premiumTransporter': 0,
	'tradegood3Price': oferta['precio'],
	'cargo_tradegood3': cantidad,
	'capacity': 5,
	'max_capacity': 5,
	'jetPropulsion': 0,
	'transporters': barcos,
	'backgroundView': 'city',
	'currentCityId': oferta['cityId'],
	'templateView': 'takeOffer',
	'currentTab': 'bargain',
	'actionRequest': s.token(),
	'ajax': 1
	}
	rta = s.post(payloadPost=data)

def do_it(s, ciudad, ofertas, cantidadAComprar, recurso):
	while cantidadAComprar > 0:
		barcosDisp = esperarLlegada(s)
		capacidad  = barcosDisp * 500
		aComprar = capacidad if capacidad < cantidadAComprar else cantidadAComprar

		for oferta in ofertas:
			if aComprar == 0:
				break
			if oferta['cantidadDisponible'] == 0:
				continue
			comprar = aComprar if oferta['cantidadDisponible'] > aComprar else oferta['cantidadDisponible']
			aComprar -= comprar
			cantidadAComprar -= comprar
			oferta['cantidadDisponible'] -= comprar
			buy(s, ciudad, oferta, comprar)



#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import time
import traceback
from ikabot.helpers.botComm import *
from ikabot.helpers.signals import setInfoSignal
from ikabot.helpers.process import forkear
from ikabot.helpers.gui import enter

def entrarDiariamente(s):
	if botValido(s) is False:
		return
	print('Se entrará todos los días automaticamente.')
	enter()

	forkear(s)
	if s.padre is True:
		return

	info = '\nEntro diariamente\n'
	setInfoSignal(s, info)
	try:
		do_it(s)
	except:
		msg = 'Error en:\n{}\nCausa:\n{}'.format(info, traceback.format_exc())
		sendToBot(msg)
	finally:
		s.logout()

def do_it(s):
	while True:
		s.get()
		time.sleep(24*60*60)
